@@ -5,7 +5,9 @@
 */
 
 #include "modelverificationpoint.h"
+#include "testhelper.h"
 
+#include <applicationcontroller.h>
 #include <reservationmanager.h>
 #include <timelinemodel.h>
 #include <tripgroupmanager.h>
@@ -28,23 +30,6 @@ Q_CONSTRUCTOR_FUNCTION(initLocale)
 class TripGroupProxyTest : public QObject
 {
     Q_OBJECT
-private:
-    void clearReservations(ReservationManager *mgr)
-    {
-        const auto batches = mgr->batches(); // copy, as this is getting modified in the process
-        for (const auto &id : batches) {
-            mgr->removeBatch(id);
-        }
-        QCOMPARE(mgr->batches().size(), 0);
-    }
-
-    QByteArray readFile(const QString &fn)
-    {
-        QFile f(fn);
-        f.open(QFile::ReadOnly);
-        return f.readAll();
-    }
-
 private Q_SLOTS:
     void initTestCase()
     {
@@ -59,10 +44,12 @@ private Q_SLOTS:
     void testExpandCollapse()
     {
         ReservationManager resMgr;
-        clearReservations(&resMgr);
-        resMgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/data/timeline/multi-traveler-merge-with-countryinfo.json")));
-        resMgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/data/google-multi-passenger-flight.json")));
-        resMgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
+        Test::clearAll(&resMgr);
+        ApplicationController ctrl;
+        ctrl.setReservationManager(&resMgr);
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/data/timeline/multi-traveler-merge-with-countryinfo.json")));
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/data/google-multi-passenger-flight.json")));
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
 
         TripGroupManager groupMgr;
         QSignalSpy addSpy(&groupMgr, &TripGroupManager::tripGroupAdded);
@@ -113,8 +100,10 @@ private Q_SLOTS:
     void testCurrentGroup()
     {
         ReservationManager resMgr;
-        clearReservations(&resMgr);
-        resMgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
+        Test::clearAll(&resMgr);
+        ApplicationController ctrl;
+        ctrl.setReservationManager(&resMgr);
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
 
         TripGroupManager groupMgr;
         QSignalSpy addSpy(&groupMgr, &TripGroupManager::tripGroupAdded);
