@@ -6,9 +6,9 @@
 
 #include "reservationmanager.h"
 #include "pkpassmanager.h"
+#include "reservationhelper.h"
 #include "logging.h"
 
-#include <KItinerary/ExtractorEngine>
 #include <KItinerary/ExtractorPostprocessor>
 #include <KItinerary/Event>
 #include <KItinerary/Flight>
@@ -18,12 +18,6 @@
 #include <KItinerary/SortUtil>
 #include <KItinerary/Visit>
 
-#include <KPkPass/Pass>
-
-#ifdef Q_OS_ANDROID
-#include <KMime/Message>
-#endif
-
 #include <KLocalizedString>
 
 #include <QDate>
@@ -55,6 +49,8 @@ static bool isSameTrip(const QVariant &lhs, const QVariant &rhs)
 ReservationManager::ReservationManager(QObject* parent)
     : QObject(parent)
 {
+    ReservationHelper::setup();
+
     m_validator.setAcceptedTypes<
         BusReservation,
         EventReservation,
@@ -72,14 +68,6 @@ ReservationManager::ReservationManager(QObject* parent)
 
 ReservationManager::~ReservationManager() = default;
 
-void ReservationManager::setPkPassManager(PkPassManager* mgr)
-{
-    m_passMgr = mgr;
-    connect(mgr, &PkPassManager::passAdded, this, &ReservationManager::passAdded);
-    connect(mgr, &PkPassManager::passUpdated, this, &ReservationManager::passUpdated);
-    connect(mgr, &PkPassManager::passRemoved, this, &ReservationManager::passRemoved);
-}
-
 bool ReservationManager::isEmpty() const
 {
     return m_batchToResMap.empty();
@@ -148,23 +136,6 @@ QString ReservationManager::batchesBasePath()
     return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QLatin1String("/batches/");
 }
 
-QVector<QString> ReservationManager::importReservation(const QByteArray& data, const QString &fileName)
-{
-    ExtractorEngine engine;
-    engine.setContextDate(QDateTime(QDate::currentDate(), QTime(0, 0)));
-    engine.setData(data, fileName);
-    return importReservations(JsonLdDocument::fromJson(engine.extract()));
-}
-
-#ifdef Q_OS_ANDROID
-QVector<QString> ReservationManager::importReservation(KMime::Message *msg)
-{
-    ExtractorEngine engine;
-    engine.setContent(QVariant::fromValue<KMime::Content*>(msg), u"message/rfc822");
-    return importReservations(JsonLdDocument::fromJson(engine.extract()));
-}
-#endif
-
 QVector<QString> ReservationManager::importReservations(const QVector<QVariant> &resData)
 {
     ExtractorPostprocessor postproc;
@@ -201,7 +172,6 @@ QVector<QString> ReservationManager::importReservations(const QVector<QVariant>
         ids.push_back(addReservation(res));
     }
 
-    Q_EMIT infoMessage(i18np("One reservation imported.", "%1 reservations imported.", ids.size()));
     return ids;
 }
 
@@ -332,27 +302,6 @@ void ReservationManager::removeBatch(const QString &batchId)
     removeReservation(batchId);
 }
 
-void ReservationManager::passAdded(const QString& passId)
-{
-    const auto pass = m_passMgr->pass(passId);
-    ExtractorEngine engine;
-    engine.setContent(QVariant::fromValue<KPkPass::Pass*>(pass), u"application/vnd.apple.pkpass");
-    const auto data = engine.extract();
-    const auto res = JsonLdDocument::fromJson(data);
-    importReservations(res);
-}
-
-void ReservationManager::passUpdated(const QString& passId)
-{
-    passAdded(passId);
-}
-
-void ReservationManager::passRemoved(const QString& passId)
-{
-    Q_UNUSED(passId)
-    // TODO
-}
-
 void ReservationManager::loadBatches()
 {
     Q_ASSERT(m_batches.empty());
