@@ -7,14 +7,18 @@
 #include "transfermanager.h"
 #include "logging.h"
 #include "favoritelocationmodel.h"
+#include "livedatamanager.h"
 #include "publictransport.h"
-#include "reservationhelper.h"
 #include "reservationmanager.h"
 #include "tripgroup.h"
 #include "tripgroupmanager.h"
 
+#include <KItinerary/BusTrip>
+#include <KItinerary/Event>
+#include <KItinerary/Flight>
 #include <KItinerary/LocationUtil>
 #include <KItinerary/Reservation>
+#include <KItinerary/TrainTrip>
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/JourneyReply>
@@ -32,7 +36,7 @@
 using namespace KItinerary;
 
 // bump this to trigger a full rescan for transfers
-enum { CurrentFullScanVerion = 1 };
+enum { CurrentFullScanVersion = 1 };
 
 TransferManager::TransferManager(QObject *parent)
     : QObject(parent)
@@ -65,9 +69,22 @@ void TransferManager::setFavoriteLocationModel(FavoriteLocationModel *favLocMode
     rescan();
 }
 
-void TransferManager::setPublicTransportManager(KPublicTransport::Manager *ptMgr)
+void TransferManager::setLiveDataManager(LiveDataManager *liveDataMgr)
 {
-    m_ptrMgr = ptMgr;
+    m_liveDataMgr = liveDataMgr;
+    connect(m_liveDataMgr, &LiveDataManager::arrivalUpdated,this, [this](const QString &resId) {
+        // update anchor time if we have a transfer for this
+        auto t = transfer(resId, Transfer::After);
+        if (t.state() == Transfer::Discarded || t.state() == Transfer::UndefinedState) {
+            return;
+        }
+
+        t.setAnchorTime(anchorTimeAfter(resId, m_resMgr->reservation(resId)));
+        addOrUpdateTransfer(t);
+
+        // TODO if there's existing transfer, check if we miss this now
+        // if so: warn and search for a new one if auto transfers are enabled
+    });
 }
 
 void TransferManager::setAutoAddTransfers(bool enable)
@@ -147,7 +164,8 @@ bool TransferManager::canAddTransfer(const QString& resId, Transfer::Alignment a
     t.setReservationId(resId);
     t.setAlignment(alignment);
 
-    return (alignment == Transfer::Before ? checkTransferBefore(resId, res, t) : checkTransferAfter(resId, res, t)) != ShouldRemove;
+    const bool canAdd = (alignment == Transfer::Before ? checkTransferBefore(resId, res, t) : checkTransferAfter(resId, res, t)) != ShouldRemove;
+    return canAdd && t.hasLocations() && t.anchorTime().isValid();
 }
 
 Transfer TransferManager::addTransfer(const QString& resId, Transfer::Alignment alignment)
@@ -179,7 +197,7 @@ void TransferManager::rescan(bool force)
     QSettings settings;
     settings.beginGroup(QStringLiteral("TransferManager"));
     const auto previousFullScanVersion = settings.value(QLatin1String("FullScan"), 0).toInt();
-    if (!force && previousFullScanVersion >= CurrentFullScanVerion) {
+    if (!force && previousFullScanVersion >= CurrentFullScanVersion) {
         return;
     }
 
@@ -187,7 +205,7 @@ void TransferManager::rescan(bool force)
     for (const auto &batchId : m_resMgr->batches()) {
         checkReservation(batchId);
     }
-    settings.setValue(QStringLiteral("FullScan"), CurrentFullScanVerion);
+    settings.setValue(QStringLiteral("FullScan"), CurrentFullScanVersion);
 }
 
 void TransferManager::checkReservation(const QString &resId)
@@ -199,11 +217,11 @@ void TransferManager::checkReservation(const QString &resId)
     const auto res = m_resMgr->reservation(resId);
 
     const auto now = currentDateTime();
-    if (ReservationHelper::arrivalTime(res) < now) {
+    if (anchorTimeAfter(resId, res) < now) {
         return;
     }
     checkReservation(resId, res, Transfer::After);
-    if (ReservationHelper::departureTime(res) < now) {
+    if (anchorTimeBefore(resId, res) < now) {
         return;
     }
     checkReservation(resId, res, Transfer::Before);
@@ -236,7 +254,7 @@ void TransferManager::checkReservation(const QString &resId, const QVariant &res
 
 TransferManager::CheckTransferResult TransferManager::checkTransferBefore(const QString &resId, const QVariant &res, Transfer &transfer) const
 {
-    transfer.setAnchorTime(ReservationHelper::departureTime(res));
+    transfer.setAnchorTime(anchorTimeBefore(resId, res));
     const auto isLocationChange = LocationUtil::isLocationChange(res);
     QVariant toLoc;
     if (isLocationChange) {
@@ -263,36 +281,45 @@ TransferManager::CheckTransferResult TransferManager::checkTransferBefore(const
         return notInGroup ? CanAddManually : ShouldAutoAdd;
     }
 
-    if (isLocationChange) {
-        const auto prevResId = m_resMgr->previousBatch(resId); // TODO this fails for multiple nested range elements!
-        if (prevResId.isEmpty()) {
-            return ShouldRemove;
-        }
-        const auto prevRes = m_resMgr->reservation(prevResId);
-        // TODO this needs to consider transfers before nextResId
-        QVariant prevLoc;
-        if (LocationUtil::isLocationChange(prevRes)) {
-            prevLoc = LocationUtil::arrivalLocation(prevRes);
-        } else {
-            prevLoc = LocationUtil::location(prevRes);
-        }
-        if (!toLoc.isNull() && !prevLoc.isNull() && !LocationUtil::isSameLocation(toLoc, prevLoc, LocationUtil::WalkingDistance)) {
-            qDebug() << res << prevRes << LocationUtil::name(toLoc) << LocationUtil::name(prevLoc);
-            transfer.setFrom(PublicTransport::locationFromPlace(prevLoc, prevRes));
-            transfer.setFromName(LocationUtil::name(prevLoc));
-            return ShouldAutoAdd;
+    const auto prevResId = m_resMgr->previousBatch(resId); // TODO this fails for multiple nested range elements!
+    if (prevResId.isEmpty()) {
+        return ShouldRemove;
+    }
+    const auto prevRes = m_resMgr->reservation(prevResId);
 
+    // check if there is a transfer after prevRes already
+    const auto prevTransfer = this->transfer(prevResId, Transfer::After);
+    if (prevTransfer.state() != Transfer::UndefinedState && prevTransfer.state() != Transfer::Discarded) {
+        if (prevTransfer.floatingLocationType() == Transfer::FavoriteLocation) {
+            transfer.setFrom(prevTransfer.to());
+            transfer.setFromName(prevTransfer.toName());
+            transfer.setFloatingLocationType(Transfer::FavoriteLocation);
+            return CanAddManually;
         }
+        return ShouldRemove;
     }
 
-    // TODO
+    QVariant prevLoc;
+    if (LocationUtil::isLocationChange(prevRes)) {
+        prevLoc = LocationUtil::arrivalLocation(prevRes);
+    } else {
+        prevLoc = LocationUtil::location(prevRes);
+    }
+    if (!toLoc.isNull() && !prevLoc.isNull() && !LocationUtil::isSameLocation(toLoc, prevLoc, LocationUtil::WalkingDistance)) {
+        qDebug() << res << prevRes << LocationUtil::name(toLoc) << LocationUtil::name(prevLoc) << transfer.anchorTime();
+        transfer.setFrom(PublicTransport::locationFromPlace(prevLoc, prevRes));
+        transfer.setFromName(LocationUtil::name(prevLoc));
+        transfer.setFloatingLocationType(Transfer::Reservation);
+        return isLocationChange ? ShouldAutoAdd : CanAddManually;
+
+    }
 
     return ShouldRemove;
 }
 
 TransferManager::CheckTransferResult TransferManager::checkTransferAfter(const QString &resId, const QVariant &res, Transfer &transfer) const
 {
-    transfer.setAnchorTime(ReservationHelper::arrivalTime(res));
+    transfer.setAnchorTime(anchorTimeAfter(resId, res));
     const auto isLocationChange = LocationUtil::isLocationChange(res);
     QVariant fromLoc;
     if (isLocationChange) {
@@ -315,33 +342,41 @@ TransferManager::CheckTransferResult TransferManager::checkTransferAfter(const Q
         const auto f = pickFavorite(fromLoc, resId, Transfer::After);
         transfer.setTo(locationFromFavorite(f));
         transfer.setToName(f.name());
-        transfer.setToName(i18n("Home"));
         transfer.setFloatingLocationType(Transfer::FavoriteLocation);
         return notInGroup ? CanAddManually : ShouldAutoAdd;
     }
 
-    if (isLocationChange) {
-        const auto nextResId = m_resMgr->nextBatch(resId);
-        if (nextResId.isEmpty()) {
-            return ShouldRemove;
-        }
-        // TODO this needs to consider transfers after nextResId
-        const auto nextRes = m_resMgr->reservation(nextResId);
-        QVariant nextLoc;
-        if (LocationUtil::isLocationChange(nextRes)) {
-            nextLoc = LocationUtil::departureLocation(nextRes);
-        } else {
-            nextLoc = LocationUtil::location(nextRes);
-        }
-        if (!fromLoc.isNull() && !nextLoc.isNull() && !LocationUtil::isSameLocation(fromLoc, nextLoc, LocationUtil::WalkingDistance)) {
-            qDebug() << res << nextRes << LocationUtil::name(fromLoc) << LocationUtil::name(nextLoc);
-            transfer.setTo(PublicTransport::locationFromPlace(nextLoc, nextRes));
-            transfer.setToName(LocationUtil::name(nextLoc));
-            return ShouldAutoAdd;
+    const auto nextResId = m_resMgr->nextBatch(resId);
+    if (nextResId.isEmpty()) {
+        return ShouldRemove;
+    }
+    const auto nextRes = m_resMgr->reservation(nextResId);
+
+    // check if there is a transfer before nextRes already
+    const auto nextTransfer = this->transfer(nextResId, Transfer::Before);
+    if (nextTransfer.state() != Transfer::UndefinedState && nextTransfer.state() != Transfer::Discarded) {
+        if (nextTransfer.floatingLocationType() == Transfer::FavoriteLocation) {
+            transfer.setTo(nextTransfer.from());
+            transfer.setToName(nextTransfer.fromName());
+            transfer.setFloatingLocationType(Transfer::FavoriteLocation);
+            return CanAddManually;
         }
+        return ShouldRemove;
     }
 
-    // TODO
+    QVariant nextLoc;
+    if (LocationUtil::isLocationChange(nextRes)) {
+        nextLoc = LocationUtil::departureLocation(nextRes);
+    } else {
+        nextLoc = LocationUtil::location(nextRes);
+    }
+    if (!fromLoc.isNull() && !nextLoc.isNull() && !LocationUtil::isSameLocation(fromLoc, nextLoc, LocationUtil::WalkingDistance)) {
+        qDebug() << res << nextRes << LocationUtil::name(fromLoc) << LocationUtil::name(nextLoc) << transfer.anchorTime();
+        transfer.setTo(PublicTransport::locationFromPlace(nextLoc, nextRes));
+        transfer.setToName(LocationUtil::name(nextLoc));
+        transfer.setFloatingLocationType(Transfer::Reservation);
+        return isLocationChange ? ShouldAutoAdd : CanAddManually;
+    }
 
     return ShouldRemove;
 }
@@ -395,6 +430,62 @@ void TransferManager::determineAnchorDeltaDefault(Transfer &transfer, const QVar
     }
 }
 
+QDateTime TransferManager::anchorTimeBefore(const QString &resId, const QVariant &res) const
+{
+    if (JsonLd::isA<TrainReservation>(res)) {
+        const auto departure = m_liveDataMgr->departure(resId);
+        if (departure.hasExpectedDepartureTime()) {
+            return departure.expectedDepartureTime();
+        }
+        return res.value<TrainReservation>().reservationFor().value<TrainTrip>().departureTime();
+    }
+    if (JsonLd::isA<BusReservation>(res)) {
+        return res.value<BusReservation>().reservationFor().value<BusTrip>().departureTime();
+    }
+    if (JsonLd::isA<FlightReservation>(res)) {
+        const auto flight = res.value<FlightReservation>().reservationFor().value<Flight>();
+        if (flight.boardingTime().isValid()) {
+            return flight.boardingTime();
+        }
+        return flight.departureTime();
+    }
+    if (JsonLd::isA<EventReservation>(res)) {
+        const auto event = res.value<EventReservation>().reservationFor().value<Event>();
+        if (event.doorTime().isValid()) {
+            return event.doorTime();
+        }
+        return event.startDate();
+    }
+    if (JsonLd::isA<FoodEstablishmentReservation>(res)) {
+        return res.value<FoodEstablishmentReservation>().startTime();
+    }
+    return {};
+}
+
+QDateTime TransferManager::anchorTimeAfter(const QString &resId, const QVariant &res) const
+{
+    if (JsonLd::isA<TrainReservation>(res)) {
+        const auto arrival = m_liveDataMgr->arrival(resId);
+        if (arrival.hasExpectedArrivalTime()) {
+            return arrival.expectedArrivalTime();
+        }
+        return res.value<TrainReservation>().reservationFor().value<TrainTrip>().arrivalTime();
+    }
+    if (JsonLd::isA<BusReservation>(res)) {
+        return res.value<BusReservation>().reservationFor().value<BusTrip>().arrivalTime();
+    }
+    if (JsonLd::isA<FlightReservation>(res)) {
+        return res.value<FlightReservation>().reservationFor().value<Flight>().arrivalTime();
+    }
+    if (JsonLd::isA<EventReservation>(res)) {
+        return res.value<EventReservation>().reservationFor().value<Event>().endDate();
+    }
+    if (JsonLd::isA<FoodEstablishmentReservation>(res)) {
+        return res.value<FoodEstablishmentReservation>().endTime();
+    }
+    return {};
+}
+
 KPublicTransport::Location TransferManager::locationFromFavorite(const FavoriteLocation &favLoc)
 {
     KPublicTransport::Location loc;
@@ -405,19 +496,29 @@ KPublicTransport::Location TransferManager::locationFromFavorite(const FavoriteL
 
 FavoriteLocation TransferManager::pickFavorite(const QVariant &anchoredLoc, const QString &resId, Transfer::Alignment alignment) const
 {
+    const auto &favLocs = m_favLocModel->favoriteLocations();
+    if (favLocs.empty()) {
+        return {};
+    }
+
     // TODO selection strategy:
     // (1) pick the same favorite as was used before/after resId
     // (2) pick the favorite closest to anchoredLoc - this can work very well if the favorites aren't close to each other
     // (3) pick the first one
 
-    Q_UNUSED(anchoredLoc)
     Q_UNUSED(resId)
     Q_UNUSED(alignment)
 
-    if (m_favLocModel->rowCount() == 0) {
-        return {};
+    // pick the first location within a 50km distance
+    const auto anchordCoord = LocationUtil::geo(anchoredLoc);
+    const auto it = std::find_if(favLocs.begin(), favLocs.end(), [&anchordCoord](const auto &fav) {
+        const auto d = LocationUtil::distance(anchordCoord.latitude(), anchordCoord.longitude(), fav.latitude(), fav.longitude());
+        return d < 50'000;
+    });
+    if (it != favLocs.end()) {
+        return (*it);
     }
-    return m_favLocModel->favoriteLocations()[0];
+    return {};
 }
 
 void TransferManager::addOrUpdateTransfer(Transfer &t)
@@ -518,8 +619,7 @@ static KPublicTransport::Journey pickJourney(const Transfer &t, const std::vecto
     if (journeys.empty()) {
         return {};
     }
-    Q_UNUSED(t)
-    return journeys[0]; // TODO
+    return t.alignment() == Transfer::Before ? journeys.back() : journeys.front();
 }
 
 void TransferManager::autoFillTransfer(Transfer &t)
@@ -530,7 +630,7 @@ void TransferManager::autoFillTransfer(Transfer &t)
 
     t.setState(Transfer::Searching);
 
-    auto reply = m_ptrMgr->queryJourney(journeyRequestForTransfer(t));
+    auto reply = m_liveDataMgr->publicTransportManager()->queryJourney(journeyRequestForTransfer(t));
     const auto batchId = t.reservationId();
     const auto alignment = t.alignment();
     connect(reply, &KPublicTransport::JourneyReply::finished, this, [this, reply, batchId, alignment]() {
